/*ParseGrain parse parameters on grains command */
#include <string.h>
#include "cddefines.h"
#include "grainvar.h"
#include "physok.h"
#include "input.h"
#include "dheton.h"
#include "ffmtread.h"
#include "lgmatch.h"
#include "rdfile.h"
#include "getquote.h"
#include "parse.h"

void ParseGrain(char *chCard, 
  int *lgDset)
{
	int lgEOL, 
	  lgEOL1, 
	  lgLinSet, 
	  lgLogOn, 
	  lgVaryAbun;

	char chFile[200];/*possible name of input file with opacities */
	long int i, 
	  ipnt;
	double dep, 
	  dep1;

#	ifdef DEBUG_FUN
	fputs( "<+>ParseGrain()\n", debug_fp );
#	endif

	/* grain model, numbers are logs of grain abundance rel to std */
	GrainVar.lgDustOn = TRUE;
	*lgDset = TRUE;

	/* log - linear option for grains */
	lgLogOn = FALSE;
	lgLinSet = FALSE;
	if( lgMatch(" LOG",chCard) )
	{
		lgLogOn = TRUE;
		lgLinSet = FALSE;
	}

	else if( lgMatch("LINE",chCard) )
	{
		lgLogOn = TRUE;
		lgLinSet = TRUE;
	}

	/* check for the keyword "function" - this is a special case */
	if( lgMatch("FUNC",chCard) )
	{
		lgVaryAbun = TRUE;
	}
	else
	{
		lgVaryAbun = FALSE;
	}

	/* get the grain abundance as the first parameter */
	i = 5;
	dep = FFmtRead(chCard,&i,76,&lgEOL);

	/* was keyword log on the line? */
	if( lgLogOn )
	{
		/* log or linear was specified, which was it */
		if( lgLinSet )
		{
			/* linear quantity enterd, and we want a log */
			if( dep > 0. )
			{
				dep = log10(dep);
			}
			else
			{
				fprintf( ioQQQ, " Impossible for linear negative abundance.\n" );
				fprintf( ioQQQ, " Abundance entered was%10.2e\n", dep );
				fprintf( ioQQQ, " Sorry.\n" );
				puts( "[Stop in getgrain]" );
				exit(1);
			}
		}
	}

	else
	{
		/* neither log nor linear specified, check sign
		 * force it to be a log - linear if greater than 0 */
		if( dep > 0. )
		{
			dep = log10(dep);
		}
	}

	/* now change to linear since will be used throughout code */
	dep = pow(10.,dep);

	/* it is possible that there is a file name on the command line - 
	 * if so then we want to call correct reoutine, and not look for keywords 
	 * the signature of a eyword is a pair of quotes - is one present?
	 */
	if( strchr( input.chOrgCard,'\"' ) != NULL )
	{
		/* 
		 * this will both scan in whatever label is inside the quotes in OrgCard, 
		 * but also remove the contents there and in chCard,
		 * so that following keywords will not trigger off it
		 */
		GetQuote( chFile , chCard );
	}
	else
	{
		/* set end of line sentinal so we know file not on line */
		chFile[0] = '\0';
	}

	/* 
	 * NB! lgDustOn1(nd) turns on grains for some species.  None of these
	 * commands turn off other grains, so successive commands will turn on
	 * more and more grains.  This is intended
	 * 
	 */
	if( lgMatch("ORIO",chCard) )
	{
		/* optional keyword ORION to use orion curves for large R grains
		 * >>chng 96 nov 30, into loop for extra K Volk grain types */
		GrainVar.lgDustOn1[2] = TRUE;
		GrainVar.lgDustOn1[3] = TRUE;
		GrainVar.dstfactor[2] = (float)dep;
		GrainVar.dstfactor[3] = (float)dep;
		/* option to have grain abundance depend on depth */
		if( lgVaryAbun )
		{
			GrainVar.lgDustVary[2] = TRUE;
			GrainVar.lgDustVary[3] = TRUE;
		}
	}

	else if( lgMatch(" PAH",chCard) )
	{
		/* >>chng 97 mar 1, added this keyworkd
		 * optional keyword  PAH to use two PAH grain species */
		GrainVar.lgDustOn1[8] = TRUE;
		GrainVar.lgDustOn1[9] = TRUE;
		GrainVar.dstfactor[8] = (float)dep;
		GrainVar.dstfactor[9] = (float)dep;

		/* for PAH grains (only) there is a qheat option for quantum heating */
		if( lgMatch("QHEA",chCard) )
		{
			GrainVar.lgQHeat[8] = TRUE;
			GrainVar.lgQHeat[9] = TRUE;
		}

		/* option to have grain abundance depend on depth */
		if( lgVaryAbun )
		{
			GrainVar.lgDustVary[8] = TRUE;
			GrainVar.lgDustVary[9] = TRUE;
		}
	}

	else if( lgMatch("GRAY",chCard) || lgMatch("GREY",chCard) )
	{
		/* >>chng 97 mar 1, added this keyworkd, 97 july 18, second spelling
		 * optional keyword GRAY to use gray grain, 0.1 micron */
		GrainVar.lgDustOn1[7] = TRUE;
		GrainVar.dstfactor[7] = (float)dep;

		/* option to have grain abundance depend on depth */
		if( lgVaryAbun )
		{
			GrainVar.lgDustVary[7] = TRUE;
		}
	}

	else if( lgMatch(" AGB",chCard) || lgMatch("AGB ",chCard) )
	{
		/* key used to be planetary nebula, should be agb
		 * planetary nebula - post agb, from Kevin Volk
		 * Try to read the second depletion factor (K Volk) */
		dep1 = FFmtRead(chCard,&i,76,&lgEOL1);
		if( lgEOL1 )
		{
			dep1 = dep;
		}

		/* >>chng 97 july 17, linear dep1
		 * if(dep1.gt.0.) dep1 = LOG10(dep1) */
		if( dep1 <= 0. )
		{
			dep1 = pow(10.,dep1);
		}

		/* End of changes here (K Volk) */
		GrainVar.lgDustOn1[0] = TRUE;
		GrainVar.lgDustOn1[6] = TRUE;
		GrainVar.dstfactor[0] = (float)dep;

		/* >>chng 96 nov 30, as per K Volk second parameter */
		GrainVar.dstfactor[6] = (float)dep1;

		/* option to have grain abundance depend on depth */
		if( lgVaryAbun )
		{
			GrainVar.lgDustVary[0] = TRUE;
			GrainVar.lgDustVary[6] = TRUE;
		}
	}

	else
	{
		/* turn one type of dust on if second number present */
		ipnt = (long int)FFmtRead(chCard,&i,76,&lgEOL);
		/* leave documented behavior or grain pointers as they were,
		 * so this is one greater than array element.  decrement here 
		 * to bring onto C scale */
		--ipnt;

		/* lgEOL if no second number was present */
		if( lgEOL )
		{
			/* no second number, use default ISM dust with optional depletion
			 * Mathis et al. ISM grains */
			GrainVar.lgDustOn1[0] = TRUE;
			GrainVar.lgDustOn1[1] = TRUE;
			GrainVar.dstfactor[0] = (float)dep;
			GrainVar.dstfactor[1] = (float)dep;

			/* option to have grain abundance depend on depth */
			if( lgVaryAbun )
			{
				GrainVar.lgDustVary[0] = TRUE;
				GrainVar.lgDustVary[1] = TRUE;
			}
		}

		/* NDUST is 20 in grainvar.h */
		else if( ipnt < 0 || ipnt >= NDUST )
		{
			fprintf( ioQQQ, " Pointer to grain type bad.  Sorry.\n" );
			puts( "[Stop in getgrain]" );
			exit(1);
		}

		else
		{
			/* this branch if grain number on line */
			GrainVar.lgDustOn1[ipnt] = TRUE;
			GrainVar.dstfactor[ipnt] = (float)dep;

			/* option to have grain abundance depend on depth */
			if( lgVaryAbun )
			{
				GrainVar.lgDustVary[ipnt] = TRUE;
			}

			/* Changes to dust for other grains (K Volk)
			 * this branch if grain species greater than 10 - we will
			 * read in optical properties from ancillary file */
			if( ipnt > 11 )
			{
				rdfile( ipnt+1 , chFile );
			}

			/* reads in parameters for the grain from a file if the
			 * number is >= 11 */

			/* Add change for quantum heating, keeping track of depletions of grain */
			GrainVar.qdep[ipnt] *= (float)pow(10.,dep);

			/* Add qheat option
			 * for grains set by number (only) there is a qheat option for
			 * quantum heating */
			if( lgMatch("QHEA",chCard) )
			{
				GrainVar.lgQHeat[ipnt] = TRUE;
			}

			/* End of changes here (K Volk) */
		}
	}

	/* option to turn off photoelectric heating by grain, NO HEATING */
	if( lgMatch("O HE",chCard) )
	{
		physok.lgPhysOK = FALSE;
		dheton.lgDHetOn = FALSE;
	}

	/* option to turn off gas cooling by grain, NO COOLING */
	if( lgMatch("O CO",chCard) )
	{
		physok.lgPhysOK = FALSE;
		dheton.lgDColOn = FALSE;
	}


#	ifdef DEBUG_FUN
	fputs( " <->ParseGrain()\n", debug_fp );
#	endif
	return;
}

