/*punt produce map of heating-cooling space for specified zone */
#include <string.h>
#include <limits.h>
#include "cddefines.h"
#ifdef EPS
#	undef EPS
#endif
#define	EPS	0.005
#include "coolants.h"
#include "mappar.h"
#include "cnegtk.h"
#include "heating.h"
#include "stopcalc.h"
#include "he.h"
#include "thigh.h"
#include "called.h"
#include "heat.h"
#include "phycon.h"
#include "zonecnt.h"
#include "h.h"
#include "tcool.h"
#include "mapr.h"
#include "trace.h"
#include "coolpr.h"
#include "pressure.h"
#include "tfidle.h"
#include "boltgn.h"
#include "nsset.h"
#include "converge.h"
#include "punt.h"

void punt(char *chType)
{
	char chLabel[5];
	char units;
	long int i, 
	  ih, 
	  j, 
	  jh, 
	  k, 
	  lambda;
	double cfrac, 
	  ch, 
	  fac, 
	  factor, 
	  hfrac, 
	  oldch, 
	  ratio, 
	  strhet, 
	  strong, 
	  t1, 
	  tlowst, 
	  tmax, 
	  tmin, 
	  torg;

#	ifdef DEBUG_FUN
	fputs( "<+>punt()\n", debug_fp );
#	endif

	t1 = phycon.te;
	torg = phycon.te;

	/* make sure pressure has been evaluated */
	strong = TotalPressure();

	/* print out all coolants if all else fails */
	if( called.lgTalk )
	{
		fprintf( ioQQQ, " Cloudy punts, Te=%10.3e HTOT=%10.3e CTOT=%10.3e nzone=%4ld\n", 
		  phycon.te, heat.htot, tcool.ctot, ZoneCnt.nzone );
		fprintf( ioQQQ, " COOLNG array is\n" );

		if( tcool.ctot > 0. )
		{
			coolpr("ZERO",1,0.,"ZERO");
			for( i=0; i < Coolants.ncltot; i++ )
			{
				ratio = Coolants.cooling[i]/tcool.ctot;
				if( ratio>EPS )
				{
					coolpr((char*)chCoolants.chClntLab[i],Coolants.collam[i],
					  ratio,"DOIT");
				}
			}

			coolpr("DONE",1,0.,"DONE");
			fprintf( ioQQQ, " Line heating array follows\n" );
			coolpr("ZERO",1,0.,"ZERO");

			for( i=0; i < Coolants.ncltot; i++ )
			{
				ratio = Coolants.heatnt[i]/tcool.ctot;
				if( ratio>EPS )
				{
					coolpr((char*)chCoolants.chClntLab[i],Coolants.collam[i],
					  ratio,"DOIT");
				}
			}

			coolpr("DONE",1,0.,"DONE" );
		}
	}

	/* map out te-ionization-cooling space before punching out. */
	if( called.lgTalk )
	{
		fprintf( ioQQQ, 
			"    Te     Heat-------------------> Cool---------------------->    dH/dT    dC/DT       Ne         NH      HII       Helium \n" );
	}

	if( strcmp(chType,"punt") == 0 )
	{
		/* this is the original use of punt, we are punting
		 * only map within factor of two of final temperature
		 * fac will the range to either side of punted temperature */
		fac = 1.5;
		tmin = torg/fac;
		tmax = torg*fac;

		/* we want about 20 steps between high and low temperature
		 * default of nMapStep is 20, set with set nmaps command */
		factor = pow(10.,log10(tmax/tmin)/(float)(MapPar.nMapStep));
		phycon.te = (float)tmin;
	}

	else if( strcmp(chType," map") == 0 )
	{
		/* create some sort of map of heating-cooling */
		tlowst = MAX2(MapPar.RangeMap[0],StopCalc.TeLowest);
		tmin = tlowst*0.998;
		tmax = MIN2(MapPar.RangeMap[1],StopCalc.TeHighest)*1.002;

		/* we want about nMapStep (=20) steps between high and low temperature */
		factor = pow(10.,log10(tmax/tmin)/(float)(MapPar.nMapStep));

		if( thigh.lgTeHigh )
		{
			/* high te */
			factor = 1./factor;
			/* TeHighest is highest possible temperature, 1E10 */
			phycon.te = (float)(MIN2(MapPar.RangeMap[1],StopCalc.TeHighest)/factor);
		}

		else
		{
			/* low te */
			phycon.te = (float)(tlowst/factor);
		}
	}

	else
	{
		/* don't know what to do */
		fprintf( ioQQQ, " PUNT called with insane argument,=%4.4s\n", 
		  chType );
		puts( "[Stop in punt]" );
		exit(1);
	}

	cnegtk.lgCNegChk = FALSE;
	mapr.nmap = 0;
	oldch = 0.;
	phycon.te *= (float)factor;

	while( phycon.te < tmax*0.999 && phycon.te > tmin*1.001 )
	{
		mapr.nmap = MIN2(MAPDIM,mapr.nmap+1);
		/* strong means nothing, but we must give value to something*/
		strong = TotalPressure();
		/* must reset upper and lower bounds for ionization distributions */
		nsset();
		/* this turns on constant reevaluation of everything */
		conv.lgSearch = TRUE;

		/* update density - temperature variables which may have changed */
		tfidle();

		/* reevaluate gaunt factors and Boltzmann factors */
		boltgn();

		/* now get inital ionizatin solution for this temperature */
		ConvEdenIoniz();
		mapr.temap[mapr.nmap-1] = phycon.te;
		mapr.cmap[mapr.nmap-1] = tcool.ctot;
		mapr.hmap[mapr.nmap-1] = heat.htot;

		lambda = 0;
		strong = 0.;

		for( j=0; j < Coolants.ncltot; j++ )
		{
			if( Coolants.cooling[j] > strong )
			{
				strcpy( chLabel, chCoolants.chClntLab[j] );
				strong = Coolants.cooling[j];
				lambda = Coolants.collam[j];
			}
		}

		cfrac = strong/tcool.ctot;
		strhet = 0.;
		/* these will be reset in following loop*/
		ih = -INT_MAX;
		jh = -INT_MAX;

		for( k=0; k < LIMELM; k++ )
		{
			for( j=0; j < LIMELM; j++ )
			{
				if( HeatingCom.heating[j][k] > strhet )
				{
					strhet = HeatingCom.heating[j][k];
					ih = k+1;
					jh = j+1;
				}
			}
		}

		ch = tcool.ctot - heat.htot;
		/* use ratio to check for change of sign since product
		 * can underflow at low densities */
		if( oldch/ch < 0. && called.lgTalk )
		{
			fprintf( ioQQQ, " ----------------------------------------------- Probable thermal solution here. --------------------------------------------\n" );
		}

		oldch = ch;
		hfrac = strhet/heat.htot;
		if( called.lgTalk )
		{
			/* convert to micros if IR transition */
			if( lambda < 100000. )
			{
				units = 'A';
			}

			else
			{
				lambda /= 10000;
				units = 'm';
			}

			if( trace.lgTrace )
			{
				fprintf( ioQQQ, "TRACE: te, htot, ctot%11.3e%11.3e%11.3e\n", 
				  phycon.te, heat.htot, tcool.ctot );
			}

			fprintf( ioQQQ, 
				"%10.4e%11.4e%4ld%4ld%6.3f%11.4e %4.4s %4ld%c%6.3f%10.2e%11.4e%11.4e%6.2f%6.2f%6.2f%6.2f\n", 
			  phycon.te, heat.htot, ih, jh, hfrac, tcool.ctot, chLabel
			  , lambda, units, cfrac, heat.dHTotDT, phycon.eden, phycon.hden, 
			  log10(MAX2(1e-9,h.hii/phycon.hden)), log10(MAX2(1e-9,
			  he.hei/he.ahe)), log10(MAX2(1e-9,he.heii/he.ahe)), log10(MAX2(1e-9,
			  he.heiii/he.ahe)) );
		}

		phycon.te *= (float)factor;
	}

	cnegtk.lgCNegChk = TRUE;
	phycon.te = (float)t1;


#	ifdef DEBUG_FUN
	fputs( " <->punt()\n", debug_fp );
#	endif
	return;
}

